#!/usr/bin/env python
# 
# Based on 2008 online
# http://code.icecube.wisc.edu/projects/icecube/browser/projects/jeb-scripts
# /releases/V00-02-00/lib/jebclient-runfilter.py
#
from I3Tray  import *
from I3Units import *
from math    import *
from os.path import expandvars
from glob    import glob
import os

workspace = expandvars("$I3_BUILD")
indir  = "/data/exp/IceCube/2008/filtered/level1/0702/"
outdir = "/net/user/mio/RealData/2008/level2/i3_files/0702/"
outrootdir = "/net/user/mio/RealData/2008/level2/root_files/0702/"
gcdfile = indir + "PFFilt_PhysicsTrig_PhysicsFiltering_Run00111266_Level1_GCD.i3.gz"
inphysfile = indir + "PFFilt_PhysicsTrig_PhysicsFiltering_Run00111266_Level1_Part00000041.i3.gz"
outphysfile = outdir + "PFFilt_PhysicsTrig_PhysicsFiltering_Run00111266_Level2_Part00000041.i3.gz"
outrootfile = outrootdir + "PFFilt_PhysicsTrig_PhysicsFiltering_Run00111266_Level2_Part00000041.root"
outbdlfile = outdir + "BadDomList.txt"

if len(sys.argv) > 1:
	gcdfile = sys.argv[1]
if len(sys.argv) > 2:
	inphysfile = sys.argv[2]
if len(sys.argv) > 3:
	outphysfile = sys.argv[3]
if len(sys.argv) > 4:
	outrootfile = sys.argv[4]
if len(sys.argv) > 5:
	outbdlfile = sys.argv[5]

load("libdataclasses")
load("libtwrclasses")
load("libdataio")
load("libDOMcalibrator")
load("libFeatureExtractor")
load("libTWRXT")
load("libTWRDistillery")
load("libAITime")
load("libicepick")
load("libDomTools")
load("libfilter-tools")
load("liblinefit")
load("libgulliver")
load("libgulliver-modules")
load("liblilliput")
load("libparaboloid")
load("libbayesian-priors")
#load("libphotonics-service")
#load("libphotorec-llh")
#load("libmue")
#load("libdouble-muon")
load("libtensor-of-inertia")
load("libcfirst")
load("libcscd-llh")
load("libicejams")
load("libtopwaveprocessor")
load("libtopeventbuilder")
load("libtoprec")
# EHE module
load("libanalysis-tree")
load("libBadDomList")
load("libophelia")
load("libportia")
load("libehe-star")

tray = I3Tray()

#
# Read Level1 i3 files
# Skip keys which are not needed by ehe-level2 test
#
tray.AddService("I3ReaderServiceFactory","reader")(
    ("FilenameList", [gcdfile,inphysfile]),
    ("SkipKeys", ["PoleIcLinefit",
		  "PoleIcLinefitParams",
		  "PoleCombinedLinefit",
		  "PoleCombinedLinefitParams",
		  "PoleCascadeLinefit",
		  "PoleCascadeLinefitParams",
		  "PoleToI",
		  "PoleToIParams",
		  "PoleTrackLlhFit",
		  "PoleTrackLlhFitFitParams",
		  "PoleInvertedTrackLlhFit",
		  "PoleInvertedTrackLlhFitFitParams",
		  "PoleTrackLlhFitOrig",
		  "PoleTrackLlhFitFitParamsOrig",
		  "PoleInvertedTrackLlhFitOrig",
		  "PoleInvertedTrackLlhFitFitParamsOrig",
		  "PoleJamsCluster",
		  "PoleJamsClusterParams",
		  "PoleJamsClusterVect",
		  "PoleJamsClusterVectParams",
		  "PoleJamsFit",
		  "PoleJamsFitParams",
		  "PoleJamsQualityFit",
		  "PoleJamsQualityFitParams",
		  "TWREventHeader0",
		  "TWREventHeader1",
		  "TWREventHeader2",
		  "TWREventHeader3",
		  "TWREventHeader4",
		  "TWREventHeader5",
		  "TWREventHeader6",
		  "TWREventHeader7",
		  "TWREventHeader8",
		  "TWREventHeader9",
		  "TWREventHeader10",
		  "TWRTriggerHierarchy0",
		  "TWRTriggerHierarchy1",
		  "TWRTriggerHierarchy2",
		  "TWRTriggerHierarchy3",
		  "TWRTriggerHierarchy4",
		  "TWRTriggerHierarchy5",
		  "TWRTriggerHierarchy6",
		  "TWRTriggerHierarchy7",
		  "TWRTriggerHierarchy8",
		  "TWRTriggerHierarchy9",
		  "TWRTriggerHierarchy10",
		  "TWRRawData0",
		  "TWRRawData1",
		  "TWRRawData2",
		  "TWRRawData3",
		  "TWRRawData4",
		  "TWRRawData5",
		  "TWRRawData6",
		  "TWRRawData7",
		  "TWRRawData8",
		  "TWRRawData9",
		  "TWRRawData10",
		  "TWRRawTimeCorrectionData0",
		  "TWRRawTimeCorrectionData1",
		  "TWRRawTimeCorrectionData2",
		  "TWRRawTimeCorrectionData3",
		  "TWRRawTimeCorrectionData4",
		  "TWRRawTimeCorrectionData5",
		  "TWRRawTimeCorrectionData6",
		  "TWRRawTimeCorrectionData7",
		  "TWRRawTimeCorrectionData8",
		  "TWRRawTimeCorrectionData9",
		  "TWRRawTimeCorrectionData10",
		  "TWRClockInfo0",
		  "TWRClockInfo1",
		  "TWRClockInfo2",
		  "TWRClockInfo3",
		  "TWRClockInfo4",
		  "TWRClockInfo5",
		  "TWRClockInfo6",
		  "TWRClockInfo7",
		  "TWRClockInfo8",
		  "TWRClockInfo9",
		  "TWRClockInfo10",
		  "TWRTWRLaunchSeriesMap0",
		  "TWRTWRLaunchSeriesMap1",
		  "TWRTWRLaunchSeriesMap2",
		  "TWRTWRLaunchSeriesMap3",
		  "TWRTWRLaunchSeriesMap4",
		  "TWRTWRLaunchSeriesMap5",
		  "TWRTWRLaunchSeriesMap6",
		  "TWRTWRLaunchSeriesMap7",
		  "TWRTWRLaunchSeriesMap8",
		  "TWRTWRLaunchSeriesMap9",
		  "TWRTWRLaunchSeriesMap10"]),
    )

tray.AddModule("I3Muxer", "multiplex")

tray.AddModule( "I3DOMLaunchCleaning", "LaunchCleaning" ) (
        ( "InIceInput",           "InIceRawData" ),
        ( "IceTopInput",          "IceTopRawData" ),
        ( "InIceOutput",          "CleanInIceRawData" ),
        ( "IceTopOutput",         "CleanIceTopRawData" ),
        ( "FirstLaunchCleaning",  True ),
        ( "CleanedKeys",          [ OMKey( 38, 59 ),          # Blackberry
				    OMKey( 39, 61 ),          # Hydrogen
				    OMKey( 39, 21 ),          # Aspudden
				    OMKey( 39, 22 ),          # Liljeholmen
				    OMKey( 40, 51 ),          # Juneberry
				    OMKey( 40, 52 ),          # Alfa_Romeo_Spider
				    OMKey( 44, 25 ),          # Mirkwood
				    OMKey( 44, 26 ),          # Pink
				    OMKey( 44, 47 ),          # Homeopatix
				    OMKey( 44, 48 ),          # Mad_Hatter
				    OMKey( 44, 55 ),          # Gaston
				    OMKey( 44, 56 ),          # Ermolai
				    OMKey( 46, 49 ),          # Gerbil
				    OMKey( 46, 50 ),          # Ringkobben
				    OMKey( 47, 55 ),          # Kemps_Card
				    OMKey( 47, 56 ),          # Ensta
				    OMKey( 53, 17 ),          # Bryozoan
				    OMKey( 53, 18 ),          # Fa
				    OMKey( 66, 33 ),          # New_York 
				    OMKey( 66, 34 ),          # Dou_Mu
				    OMKey( 66, 45 ),          # Alpaca
				    OMKey( 66, 46 ),          # Tallahassee
				    OMKey( 69, 23 ),          # Black
				    OMKey( 69, 24 ),          # Magenta
				    OMKey( 72, 37 ),          # Hundgrundet
				    OMKey( 72, 38 ),          # Buttercup
				    OMKey( 77, 47 ),          # Hamster
				    OMKey( 77, 48 ) ] )       # Meldii
	)
#
# Remove SLC-only readouts
#
tray.AddModule( "I3LCCleaning", "LCClean" ) (
        ( "InIceInput",           "CleanInIceRawData" ),
        ( "InIceOutput",          "HLCInIceRawData" )
	)

#**************************************************************
# EHE-level2 as a part of std-level2 -- begin
#**************************************************************
#
# Pick up ehe-filtered OR IceTop events
#
tray.AddService( "I3IcePickInstaller<FilterMaskFilter>", "DoEHEReco" ) (
	("FilterNameList", ["EHEFilter_08", "IceTopSTA3_08"]),
	("FilterResultName", "FilterMask"),
	)
#tray.AddModule("I3IcePickModule<FilterMaskFilter>", "skipEvents")(
#	("FilterNameList", ["EHEFilter_08", "IceTopSTA3_InIceSMT_08"]),
#	("FilterResultName", "FilterMask"),
#	#("DiscardEvents", False)
#	("DiscardEvents", True)
#	)
#
# Rename intermediate keys
#
tray.AddModule( "Rename", "Clean" ) (
        ( "Keys", [ "SummaryPulseInfo","OldSummaryPulseInfo"]),
	)
#
# Perform DOM calibration for InIce EHE events
#
tray.AddModule("I3DOMcalibrator","calibrate-inice")(
	("InputRawDataName",    "HLCInIceRawData"),
	( "OutputATWDDataName", "CalibratedEHEATWD" ),
        ( "OutputFADCDataName", "CalibratedEHEFADC" ),
        ("CorrectPedestalDroop", False),
        ( "CalibrationMode",     1),
        ( "IcePickServiceKey",   "DoEHEReco" ),
	)
#
# Portia
#
tray.AddModule("I3Portia","portiapulse")(
	("DataReadoutName",             "HLCInIceRawData"),
	("OutPortiaEventName",          "SummaryPulseInfo"),
	("MakeIceTopPulse",              False),
	("ATWDPulseSeriesName",         "ATWDPulseSeries"),
	("ATWDPortiaPulseName",         "ATWDPortiaPulse"),
	("ATWDWaveformName",            "CalibratedEHEATWD"),
	("ATWDBaseLineOption",          "eheoptimized"),
	("FADCBaseLineOption",          "eheoptimized"),
	("ATWDThresholdCharge",         0.1*I3Units.pC),
	("ATWDLEThresholdAmplitude",    0.5*I3Units.mV),
	("UseFADC", True),
	("FADCPulseSeriesName",         "FADCPulseSeries"),
	("FADCPortiaPulseName",         "FADCPortiaPulse"),
	("FADCWaveformName",            "CalibratedEHEFADC"),
	("FADCThresholdCharge",         0.1*I3Units.pC),
	("FADCLEThresholdAmplitude",    0.5*I3Units.mV),
	("MakeBestPulseSeries",         False),
        ( "IcePickServiceKey",   "DoEHEReco" ),
	)
#
# Firstguess
#
tray.AddModule("I3EHEFirstGuess","reco")(
	("InputLaunchName",        "HLCInIceRawData"),
	("MinimumNumberPulseDom",  2),
	("OutputFirstguessName",   "OpheliaFirstGuess"),
	("OutputParticleName",   "OpheliaFGParticle"),
	("InputPulseName1",        "ATWDPortiaPulse"),
	("InputPulseName2",        "FADCPortiaPulse"),
	("ChargeOption",           0),
	("LCOption",               True),
        ( "IcePickServiceKey",   "DoEHEReco" ),
	)
#
# the BadDomList service
#
xmlf = workspace + "/BadDomList/resources/scripts/QueryConfiguration.xml"
tray.AddService("I3BadDomListFactory","bdlist")(
	("QFileName", xmlf ), 
	#("BDListFile", outbdlfile), # optional output text file
	)
#
# Put BadDomList to the frame
#
tray.AddModule("I3BadDomListModule","bdmodule")(
	("BadDomsListVector","BadDomsList"),
        #( "IcePickServiceKey",   "DoEHEReco" ),
	) 
#
# Starting event analysis
#
tray.AddModule("I3EheStarLateralModule","starlat")(
	("useMCtruth",      False),
	("useRecoParticle", True),
	
	("inRecoParticleName", "OpheliaFGParticle"),
	("inAtwdPulseName", "ATWDPortiaPulse"),
	("inFadcPulseName", "FADCPortiaPulse"),
	("inBadDomListName", "BadDomsList"),
	
	("outEheStarLateralRecoName", "EheStarLateralFg"),
	("npe_zeroDom", 0.25),
	("npe_fitMin", 5),
	("npe_fillChMin", 1.0e4),
        ( "IcePickServiceKey",   "DoEHEReco" ),
	)
tray.AddModule("I3EheStarFirstDomsModule","firstdoms")(
	("inPortiaPulseName", "ATWDPortiaPulse"),
	("outEheStarFirstDomsName", "EheStarFirstDoms"),
	("npeTh", 10.0),
        ( "IcePickServiceKey",   "DoEHEReco" ),
	)
#
# Analysis-tree service
#
tray.AddService("I3RootTreeServiceFactory","service")(
	("TreeFileName", outrootfile),
	)
#
# Analysis-tree
#
#tray.AddModule("I3OpheliaConvertPortia","conv-portia")(
#	("InputPortiaEventName", "SummaryPulseInfo"),
#	("OutputPortiaEventMapName", "SummaryPulseInfoMap"),
#        #( "IcePickServiceKey",   "DoEHEReco" ),
#	)
#tray.AddModule("I3RootTreeModule","particle_tree1")(
#	("KeyNames","ShowerEnergy,OpheliaParticle"),
#        #( "IcePickServiceKey",   "DoEHEReco" ),
#	)
#tray.AddModule("I3MapStringDoubleTreeModule","map_tree")(
#	("KeyName","SummaryPulseInfoMap"),
#        #( "IcePickServiceKey",   "DoEHEReco" ),
#	)
#
# Only keep selected frames as ehe-level2 data
#
tray.AddModule( "Keep", "Cleanup" ) (
	("Keys", ["DrivingTime",
		  "I3EventHeader",
		  "I3TriggerHierarchy",
		  "JEBEventInfo",
		  "FilterMask",
		  "SyncPulseMap",
                  "InIceRawData",
		  "IceTopRawData",
		  # EHE level2
		  "BadDomsList",
		  "CalibratedEHEATWD",
		  "CalibratedEHEFADC",
		  "ATWDPortiaPulse",
		  "FADCPortiaPulse",
		  "SummaryPulseInfo",
		  "OpheliaFirstGuess",
		  "EheStarLateralFg",
		  "EheStarFirstDoms"])
	)	
#**************************************************************
# AddModule: EHE-level2 as a part of std-level2 -- end
#**************************************************************

#
# Write physics frames
#
tray.AddModule("I3Writer","physwriter")(
	("Filename", outphysfile),
	("Streams",["Physics"]),
	)

tray.AddModule("TrashCan", "bye")

tray.Execute(500)
tray.Finish()
